<!-- [QPRTemplateStart: jswsfutils.tpl] -->
<script type="text/javascript">

/*
 *  QPR Web services foundation/QPR Portal interaction class.
 *  Can be used either using only callbacks like this:
 *
 *  var wsf = WSFUtils();
 *  wsf.LoadProxy(function () {
 *
 *    wsf.AutoAuthenticate(function (sessionId) {
 *
 *      wsf.oClient.CallSomeMethod(sessionId, param1, param2,
 *                                 function(result){
 *        DoSomethinWith(result);
 *        wsf.LogOff();
 *      }, HandleError);
 *
 *    }, HandleError)
 *
 *  }, HandleError);
 *
 *
 *  or by inheriting from this class with jQuery like this:
 *
 *  function Processor() {
 *    var t; return (t = $.extend(WSFUtils(),
 *  {
 *    Go: function()
 *    {
 *      t.LoadProxy(function() { t.AutoAuthenticate(t.Process, t.HandleError); },
 *                  t.HandleError);
 *    }
 *
 *    Process: function()
 *    {
 *      t.oClient.CallSomeMethod(t.sSessionId, param1, param2,
 *                               t.HandleResult, t.HandleError);
 *    }
 *
 *    HandleResult: function(result)
 *    {
 *      DoSomethingWith(result);
 *      t.LogOff();
 *    },
 *
 *
 *    HandleError: function(voError)
 *    {
 *      var sMessage = t.sGetStringFromError(voError);
 *      Debug(sMessage);
 *      t.LogOff();
 *    }
 *  }
 *  ));}
 *
 *  Processor().Go();
 *
 *
 *  or you can even ditch the class altogether after getting the session ID
 *  and use the client object directly, but why bother?
 */
function WSFUtils() {
  var t; return (t =
{
  sHiddenFrameName: '_WSFClientFrame',
  oHiddenFrame: null,
  oClient: null,
  sSessionId: null,


  LoadProxy: function(voCallback, voErrorCallback)
  {
    if ( self.OnWSFServiceProxyLoaded ) return; // Already loading
    if ( t.oClient ) {
      voCallback(t.oClient);
      return;
    }

    var oBody = oGetBodyFromDocument(document);

    if ( !oBody ) {
      voErrorCallback('Body not found from document');
      return;
    }

    if ( !t.oHiddenFrame )
     t.oHiddenFrame = oGetNamedFrame(t.sHiddenFrameName);

    if ( !t.oHiddenFrame || !t.oHiddenFrame.oGetClient ) {
      var bSucceeded = false, oTimer;

      var oTimeout = function () {
        if ( !bSucceeded ) {
          self.OnWSFServiceProxyLoaded = null;
          voErrorCallback(sDecodeURI('<#LocalizedTextMessage id="Could not communicate with QPR Web Services Foundation.">'));
        }
      };

      var oSuccess = function (voClient) {
        bSucceeded = true;
        try { clearTimeout(oTimer); } catch ( e ) {}
        self.OnWSFServiceProxyLoaded = null;

        if ( !voClient.QPR_bMethodsWrapped ) {
          for ( var sMethod in voClient )
            voClient[sMethod] = t.oCreateFixWrapper(voClient[sMethod],
                                                    voClient);
          voClient.QPR_bMethodsWrapped = true;
        }

        t.oClient = voClient;
        t.oCreateTypedArray = t.oClient.QPR_CreateTypedArray;
        voCallback(voClient);
      }

      self.OnWSFServiceProxyLoaded = oSuccess;
      t.oHiddenFrame = oCreateHiddenFrame(oBody, t.sHiddenFrameName);
      t.oHiddenFrame.src = sGetWSFLocation() + '/serviceproxy.aspx';

      oTimer = setTimeout(oTimeout, 120000);
    } else {
      t.oClient = t.oHiddenFrame.oGetClient();
      t.oCreateTypedArray = t.oClient.QPR_CreateTypedArray;
      voCallback(t.oClient);
    }
  },


  AutoAuthenticate: function(voCallback, voErrorCallback)
  {
    $.get(sAddUIPOrSession('<#TokenGenerationURL>'), '',
          function (voData, vsStatus) {
            if ( !voData || !voData.ErrorCode )
              voErrorCallback(vsStatus);
            else if ( voData.ErrorCode != 'success' )
              voErrorCallback(voData.ErrorCode);
            else {
              t.oClient.Authenticate('qpr_token', voData.Data,
                                         function (vsSessionId) {
                                           t.sSessionId = vsSessionId;
                                           voCallback(vsSessionId);
                                         }, voErrorCallback);
            }
          }, 'json');
  },


  CallStreamMethod: function(vsMethod, voParams, voCallback, voErrorCallback)
  {
    var sURL = t.oClient.QPR_GetMethodURL(vsMethod);
    var oParams = {}, i;
    for ( i in voParams ) oParams[i] = t.oClient.QPR_Serialize(voParams[i]);
    t.oClient.QPR_SubmitFormRequest(sURL, oParams, voCallback,voErrorCallback);
  },


  LogOff: function(voCallback, voErrorCallback)
  {
    if ( t.oClient && t.sSessionId ) {
      t.oClient.LogOff(t.sSessionId,
                       voCallback ? voCallback : function () {},
                       voErrorCallback ? voErrorCallback : function () {});
      t.sSessionId = null;
    } else if ( voCallback )
      voCallback();
  },


  sGetStringFromError: function(voError)
  {
    var sMessage;

    if ( !voError )
      sMessage = 'Unknown reason';
    else if ( typeof(voError) == 'string' )
      sMessage = voError;
    else if ( typeof(voError.get_message) === 'function' )
      sMessage = voError.get_message();
    else if ( typeof(voError.Message) === 'string' )
      sMessage = voError.Message;
    else if ( voError.sStatus )
      sMessage = 'HTTP error: ' + voError.sStatus;
    else {
      sMessage = null;
      try { sMessage = $.trim($(voError).text()); } catch ( e ) {}
      if ( !sMessage || sMessage == '' )
        sMessage = 'Unknown reason';
    }
      
    return sMessage;
  },


  oSignatureRegexp: /^function(\s*)(| [a-z0-9_]*)(\s*)\((.*?)\)/i,
  oUnparsedJSON: /^\{.*"(d|ExceptionDetail)"(\s*):/,

  oCreateFixWrapper: function(voFunction, voClient) {
    function oCreateWrapper(viCallbackIndex) {
      function oSuccessWrapper(volResults, voOriginalSuccess, voError) {
        var oParsedResult;

        // In some unclear conditions, at least back in .Net 3.5, WCF sometimes
        // manages to return unparsed results and/or results with incorrect HTTP
        // status codes. This wrapper checks for those cases.
        // TODO: See if the condition still exists in .Net 4.0.
        if ( typeof(volResults[0]) !== 'string' ||
             !t.oUnparsedJSON.test(volResults[0]) )
          try {
            return voOriginalSuccess.apply(this, volResults);
          } catch ( e ) {
            // In IE, closing an iframe while another frame is doing
            // XMLHttpRequests might cause the success and error callbacks to
            // get free'd before the request returns. Eat these errors
            // silently.
            return;
          }

        try {
          oParsedResult = eval('(' + volResults[0] + ')');

          for ( var i in oParsedResult )
            if ( i == 'ExceptionDetail' )
              try {
                return voError.call(this, oParsedResult);
              } catch ( e ) {}

          try {
            return voOriginalSuccess.call(this, oParsedResult.d);
          } catch ( e ) {}
        } catch ( e ) {
          // Should be impossible
          t.HandleError('Could not parse web services response: ' +
                        volResults[0] + "\r\nException:\r\n" + e);
          return;
        }
      }

      // Create a wrapper function that inherits from the ASPX file's type hierarchy.
      // Since .Net 4.0, the ASPX client has started checking
      // these types under certain conditions.
      function oCreateTypedFunction(voCallback) {
        var oWrapper = voClient.QPR_CreateFunction('',
          'return arguments.callee.poCallback.apply(this, arguments);');
        oWrapper.poCallback = voCallback;
        return oWrapper;
      }

      return function() {
        var olArguments, oOriginalSuccess;

        if ( typeof(arguments[viCallbackIndex]) !== 'function' ||
             typeof(arguments[viCallbackIndex + 1]) !== 'function' )
          return voFunction.apply(this, arguments);

        oOriginalSuccess = arguments[viCallbackIndex];

        olArguments = Array.apply(this, arguments);
        olArguments[viCallbackIndex] = oCreateTypedFunction(function() {
          return oSuccessWrapper.call(this, Array.apply(null, arguments),
                                           oOriginalSuccess,
                                           olArguments[viCallbackIndex + 1]);
        });

        olArguments[viCallbackIndex + 1] =
          oCreateTypedFunction(olArguments[viCallbackIndex + 1]);

        return voFunction.apply(this, olArguments);
      };
    }

    var slMatches, sSignature, slParams;
    if ( typeof(voFunction) !== 'function' ) return voFunction;

    slMatches = t.oSignatureRegexp.exec('' + voFunction);
    if ( !slMatches || !slMatches[4] )
      return voFunction;
    sSignature = slMatches[4];
    slParams = $.map(sSignature.split(','), $.trim);

    for ( var i = 0; i < slParams.length; i++ )
      if ( slParams[i] == 'succeededCallback' &&
           slParams[i + 1] == 'failedCallback' )
        return oCreateWrapper(i);

    return voFunction;
  },


  // Overridden later
  oCreateTypedArray: function(volArray)
  {
    return volArray;
  }
}
  );
}

</script>
<!-- [QPRTemplateEnd: jswsfutils.tpl] -->
